﻿using System.Threading.Tasks;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Mapping;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using orbitgt.mapping3d.sdk.viewer;

namespace plugin_3dm_arcgispro
{
    /// <summary>
    /// Class ToolPickLocation defines a tool that lets the user pick a position on a map view and send it to the 3D Mapping viewer
    /// </summary>
    internal class ToolPickLocation : MapTool
    {
        static public string DAML_ID = "plugin_3dm_arcgispro_ToolPickLocation";
        static public ToolPickLocation INSTANCE = null;

        internal bool pickPositionMode = false; // either "pick-position" or "focus" (default) mode

        public ToolPickLocation()
        {
            IsSketchTool = false; // no sketching
            SketchType = SketchGeometryType.Rectangle;
            SketchOutputMode = SketchOutputMode.Map;
            INSTANCE = this;
        }

        protected override void OnToolMouseDown(MapViewMouseButtonEventArgs e)
        {
            if (e.ChangedButton == System.Windows.Input.MouseButton.Left) e.Handled = true;
        }

        protected override Task HandleMouseDownAsync(MapViewMouseButtonEventArgs e)
        {
            return QueuedTask.Run(() =>
            {
                // we should have a viewer
                if ((PluginArcGIS.INSTANCE != null) && PluginArcGIS.INSTANCE.isReady())
                {
                    // get the click position in the viewer CRS
                    MapPoint point = MapView.Active.ClientToMap(e.ClientPoint);
                    MapPoint viewerPoint = GeometryEngine.Instance.Project(point, MapObjects.VIEWER_CRS) as MapPoint;
                    // make the viewer focus to the new position
                    Viewer3DM viewer = PluginArcGIS.INSTANCE.getViewer();
                    Coordinate position = Coordinate.fromXY("" + MapObjects.VIEWER_CRS_CODE, viewerPoint.X, viewerPoint.Y);
                    if (pickPositionMode) viewer.setPickPosition(position);
                    else viewer.setFocusPosition(position);
                }
                // clear the pick-position mode
                pickPositionMode = false;
            });
        }
    }
}
